type4projectileHandler = {
	shellNum = 1,
	shells = {},
	defaultShell = {active = false},
}

function init()
	getmodel = GetFloat("savegame.mod.getmodel")
	toolmodel = {}
	toolmodel = "MOD/vox/type4.vox"
	
	RegisterTool("type4", "Type 4 Arisaka", toolmodel)
	SetBool("game.tool.type4.enabled", true)
	SetFloat("game.tool.type4.ammo", 101)
	
	unlimitedammo = GetBool("savegame.mod.unlimitedammo")
	piercingammo = GetBool("savegame.mod.piercingammo")
	
	magsize = 10
	magcount = 20
	

	damage = 0.95
	gravity = Vec(0, 0, 0)
	velocity = 1.64
	reloadTime = 1.2
	shotDelay = 60/192
	spreadTimer = 0
	cockTimer = 0
	ammo = magsize
	mags = magcount
	reloading = false
	empty = false
	
	for i=1, ammo do
		type4projectileHandler.shells[i] = deepcopy(type4projectileHandler.defaultShell)
	end

	shootTimer = 0
	reloadTimer = 0
	recoilTimer = 0
	lightTimer = 0
	magoutTimer = 0
	maginTimer = 0
	meleeTimer = 0
	
	gunsound = LoadSound("MOD/snd/type4fire.ogg")
	cocksound = LoadSound("MOD/snd/slide.ogg")
	reloadsound = LoadSound("MOD/snd/magout.ogg")
	maginsound = LoadSound("MOD/snd/magin.ogg")
	dryfiresound = LoadSound("MOD/snd/dryfire.ogg")
	refillsound = LoadSound("MOD/snd/refill.ogg")
	lastshotsound = LoadSound("MOD/snd/lastshot.ogg")
	bayonet = LoadSound("MOD/snd/bayonet.ogg")
end

function canShoot()
	local vehicle = GetPlayerVehicle()
	if vehicle ~= 0 then
		local driverPos = GetVehicleDriverPos(vehicle)
		local t = GetVehicleTransform(vehicle)
		local worldPos = TransformToParentPoint(t, driverPos)
		local cameraPos = GetCameraTransform().pos
		local length = VecLength(VecSub(cameraPos, worldPos))
		if length < 1 then
			return true
		else
			return false
		end
	end
	if meleeTimer > 0 then
		return false
	elseif reloadTimer > 0 then
		return false
	else
	return true
	end
end

function deepcopy(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in next, orig, nil do
            copy[deepcopy(orig_key)] = deepcopy(orig_value)
        end
        setmetatable(copy, deepcopy(getmetatable(orig)))
    else
        copy = orig
    end
    return copy
end

function GetAimPos()
	local ct = GetCameraTransform()
	local forwardPos = TransformToParentPoint(ct, Vec(0, 0, -100))
    local direction = VecSub(forwardPos, ct.pos)
    local distance = VecLength(direction)
	local direction = VecNormalize(direction)
	local hit, hitDistance = QueryRaycast(ct.pos, direction, distance)
	if hit then
		forwardPos = TransformToParentPoint(ct, Vec(0, 0, -hitDistance))
		distance = hitDistance
	end
	return forwardPos, hit, distance
end

function Shoot()
	if shootTimer > 0 or ammo == 0 then
		return
	end
	
	aimpos, hit, distance = GetAimPos()

	local p = toolPos
	local dir = VecSub(aimpos, p)
	local maxSpread = InputDown("ctrl") and 1 or 1.2
	local spread = math.min(spreadTimer, maxSpread) * distance/150
	dir[1] = dir[1] + (math.random()-0.5)*2*spread
	dir[2] = dir[2] + (math.random()-0.5)*2*spread
	dir[3] = dir[3] + (math.random()-0.5)*2*spread
	
	type4projectileHandler.shells[type4projectileHandler.shellNum] = deepcopy(type4projectileHandler.defaultShell)
	loadedShell = type4projectileHandler.shells[type4projectileHandler.shellNum] 
	loadedShell.active = true
	loadedShell.pos = p
	loadedShell.predictedBulletVelocity = VecScale(dir, velocity*(100/distance))
	type4projectileHandler.shellNum = (type4projectileHandler.shellNum%#type4projectileHandler.shells) + 1
	
	SpawnParticle("fire", toolPos, Vec(0, 1.3+math.random(1,10)*0.1, 0), 0.3, 1)
	PlaySound(gunsound, GetPlayerTransform().pos, 0.9, false)

	if not unlimitedammo then
		ammo = ammo - 1
	end
	shootTimer = shotDelay
	recoilTimer = shotDelay
	lightTimer = shotDelay/2
	spreadTimer = spreadTimer + 1.25
end

function Bayonet()
	maxDist = 6
	local t = GetCameraTransform()
	local fwd = VecNormalize(TransformToParentVec(t, Vec(0, 0, -1)))
	local hit, dist = QueryRaycast(t.pos, fwd, maxDist)
	if not hit then
		dist = maxDist
	end
	local hitPos = VecAdd(t.pos, VecScale(fwd, dist))
	if hit then
		MakeHole(hitPos, 0.4, 0.3)
	end
	PlaySound(bayonet,GetPlayerTransform().pos,0.9, false)
	meleeTimer = 1
end

function ProjectileOperations(projectile)
	projectile.predictedBulletVelocity = VecAdd(projectile.predictedBulletVelocity, (VecScale(gravity, GetTimeStep())))
	local point2 = VecAdd(projectile.pos, VecScale(projectile.predictedBulletVelocity, GetTimeStep()))
	local dir = VecNormalize(VecSub(point2, projectile.pos))
	local hit, dist = QueryRaycast(projectile.pos, dir, VecLength(VecSub(point2, projectile.pos)))
	
	local hitTransparant, distT = QueryRaycast(projectile.pos, dir, dist * 2, 0, false)
	local hitNonTransparant, distNonT = QueryRaycast(projectile.pos, dir, dist * 2, 0, true)
	local hitWindow = hitTransparant and not hitNonTransparant
	
	if hit and hitWindow then
		local hitpoint = VecAdd(projectile.pos, VecScale(dir, distT))
		MakeHole(hitpoint, damage)
	end
	
	if hit and not hitWindow then	
		hitPos = VecAdd(projectile.pos, VecScale(VecNormalize(VecSub(point2, projectile.pos)), dist))
		projectile.active = false
		
		if not hitWindow then
			if piercingammo then
				MakeHole(hitPos, damage*0.33, damage*0.1)
			else
				MakeHole(hitPos, 0.1, 0.05)
			end
		end
	SpawnParticle("smoke", hitPos, Vec(0, 1.0+math.random(1,5)*-0.1, 0), damage, 1)
	else
		DrawLine(projectile.pos, point2)
	end
	projectile.pos = point2
end

function Reload()
	if reloading then
		return
	end
	reloading = true
	PlaySound(reloadsound, GetPlayerTransform().pos, 0.4, false)
	reloadTimer = reloadTime
	magoutTimer = reloadTime/2
	mags = mags - 1
end

function tick(dt)
	if GetString("game.player.tool") == "type4" then
		local b = GetToolBody()
		if b ~= 0 then
			local offset = Transform(Vec(0.05, -1.4, 0.5))
			local magtimer = magoutTimer + maginTimer
			local x, y, z, rot = 0.05, -1.4, 0.5, 0
			boltoffset = Vec(0,0,0)
			magoffset = Vec(2, 1, 1)
			
			if magtimer > 0 then
				offset.rot = QuatEuler(15, 0, -5)
				offset.pos = VecAdd(offset.pos, Vec(-0.2, -0.2, -0.2))
				magoffset = Vec(0, 0, 0)
				boltoffset = Vec(0, 0, 0.3)
			end
			
			SetToolTransform(offset)
			toolTrans = GetBodyTransform(b)
			toolPos = TransformToParentPoint(toolTrans, Vec(0.3, 1.4, -2.3))
			
			if recoilTimer > 0 then
				local t = Transform()
				t.pos = Vec(x, y, recoilTimer+z)
				ironrot = -recoilTimer*20-rot
				t.rot = QuatEuler(-ironrot, 0, 0)
				SetToolTransform(t)
				boltoffset = Vec(0, 0, 0.25)
				if recoilTimer < 0.01 then
					cockTimer = 0.2
				end
				recoilTimer = recoilTimer - dt
			end
			
			if meleeTimer > 0 then
				local t = Transform()
				t.pos = Vec(x, y, meleeTimer*-0.5+z)
				SetToolTransform(t)
				meleeTimer = meleeTimer - dt
			end
			if meleeTimer < 0 then
				meleeTimer = 0
			end
			
			local shelloffset = Vec(0, 0, 0)
			if cockTimer > 0 then
				cockTimer = cockTimer - dt
				shelloffset = Vec(0.4-cockTimer, -0.1-cockTimer*math.sin(math.random(2)*-1), 0.2-cockTimer*1.5)
			end

			if lightTimer > 0 then
				PointLight(toolPos, 1, 1, 1, 0.7)
				lightTimer = lightTimer - dt
			end
			
			if magoutTimer > 0 then
				magoffset = Vec(0, 0, 0)
				boltoffset = Vec(0, 0, 0.3)
				magoutTimer = magoutTimer - dt
				if magoutTimer < 0 then
					maginTimer = reloadTime/2
					PlaySound(maginsound, GetPlayerTransform().pos, 1, false)
				end
			end
			
			if maginTimer > 0 then
				magoffset = Vec(0.1-maginTimer/2, maginTimer*0.8, -0.10+maginTimer*0.5)
				maginTimer = maginTimer - dt
				--if maginTimer < 0 then
				--end
			end
			
			if ammo == 0 then
				boltoffset = Vec(0,0,0.3)
			end
			
			if body ~= b then
				body = b
				local shapes = GetBodyShapes(b)
				mag = shapes[3]
				bolt = shapes[2]
				shell = shapes[4]
				magTrans = GetShapeLocalTransform(mag)
				boltTrans = GetShapeLocalTransform(bolt)
				shellTrans = GetShapeLocalTransform(shell)
			end

			mt = TransformCopy(magTrans)
			mt.pos = VecAdd(mt.pos, magoffset)
			mt.rot = QuatRotateQuat(mt.rot, QuatEuler(-magtimer, 0, 0))
			SetShapeLocalTransform(mag, mt)
			
			bt = TransformCopy(boltTrans)
			bt.pos = VecAdd(bt.pos, boltoffset)
			SetShapeLocalTransform(bolt, bt)
			
			local cockinv = 0.4 - cockTimer

			st = TransformCopy(shellTrans)
			st.pos = VecAdd(st.pos, shelloffset)
			st.rot = QuatRotateQuat(st.rot, QuatEuler(0, -cockinv*250, -cockinv*500))
			SetShapeLocalTransform(shell, st)
		end

		if not unlimitedammo then
			if ammo < magsize and mags > 1 and InputPressed("R") then
				Reload()
			end
			
			if GetBool("ammobox.refill") then
				SetBool("ammobox.refill", false)
				mags = mags + 1
				PlaySound(refillsound, GetPlayerTransform().pos, 0.7, false)
			end

			if reloading then
				reloadTimer = reloadTimer - dt
				if reloadTimer < 0 then
					ammo = magsize
					reloadTimer = 0
					PlaySound(cocksound)
					reloading = false
					empty = false
				end
			end
		end
		
		if InputPressed("lmb") and canShoot() then
				Shoot()
				end
		end
		
		if InputPressed("lmb") and ammo == 0 and not reloading and not empty then
			Shoot()
			SpawnParticle("darksmoke", toolPos, Vec(0, 1.3+math.random(1,10)*0.1, 0), 0, 0.5)
			PlaySound(lastshotsound, GetPlayerTransform().pos, 0.7, false)
			empty = true
		end

		if InputPressed("lmb") and not reloading then
			spreadTimer = 0
			if ammo == 0 then
				PlaySound(dryfiresound, GetPlayerTransform().pos, 0.8, false)
			end
		end
		
		if InputPressed("X") and canShoot() == true and meleeTimer == 0 then
			Bayonet()
		end
		
		if shootTimer > 0 or ammo == 0 then
				shootTimer = shootTimer - dt
		end
		for key, shell in ipairs(type4projectileHandler.shells) do
			if shell.active then
				ProjectileOperations(shell)
			end
	end		
end

function draw()
	if GetString("game.player.tool") == "type4" and GetPlayerVehicle() == 0 and not unlimitedammo then
		UiPush()
			UiTranslate(UiCenter(), UiHeight()-60)
			UiAlign("center middle")
			UiColor(1, 1, 1)
			UiFont("bold.ttf", 32)
			UiTextOutline(0,0,0,1,0.1)
			if reloading then
				UiText("Reloading")
			else
				UiText(ammo.."/"..magsize*math.max(0, mags-1))
			end
		UiPop()
	end
end